#!/usr/bin/env python3
import sys
import time
import logging
import argparse
import toml
import logging.handlers
from pathlib import Path
from watchdog.events import FileSystemEventHandler
from watchdog.observers import Observer


class Config:
    size = 10000

    def __init__(self):
        try:
            dict_toml = toml.load(Path.home().joinpath("observer.toml"))
            size = int(dict_toml["LOG"]["size"])
            self.size = size if size >= 1 else 10000
        except (FileNotFoundError, toml.decoder.TomlDecodeError) as e:
            print(e)


class Logger:
    @classmethod
    def logger(cls, module_name):
        logger = logging.getLogger(module_name)
        rh = logging.handlers.RotatingFileHandler(r'/tmp/app.log',
                                                  maxBytes=Config().size,
                                                  backupCount=5)
        formatter = logging.Formatter(
            '%(asctime)s:%(levelname)s-%(filename)s-%(funcName)s-%(message)s')
        rh.setFormatter(formatter)
        logger.addHandler(rh)
        logger.setLevel(logging.DEBUG)
        return logger


class EventHandler(FileSystemEventHandler):
    def __init__(self):
        self.logger = Logger.logger(self.__class__.__name__)

    def on_created(self, event):  # ファイル／ディレクトリ作成時
        self.logger.info("Created " + event.src_path)

    def on_deleted(self, event):  # ファイル／ディレクトリ削除時
        self.logger.info("Deleted " + event.src_path)

    def on_modified(self, event):  # ファイル／ディレクトリ修正時
        self.logger.info("Modified " + event.src_path)

    def on_moved(self, event):  # ファイル／ディレクトリ移動時
        self.logger.info("Moved " + event.src_path)


class Main():
    def run(self, target):
        logger = Logger.logger(self.__class__.__name__)
        observer = Observer()
        # イベントハンドラ、監視対象ディレクトリ、再帰的に監視するかを指定
        observer.schedule(EventHandler(), target, recursive=True)

        logger.debug("[Start] Observe " + target)
        observer.start()  # 監視開始

        try:
            while True:
                time.sleep(1)
        except KeyboardInterrupt:
            observer.stop()  # 監視停止
        observer.join()  # スレッド停止待ち


if __name__ == '__main__':
    logger = Logger.logger(__name__)

    parser = argparse.ArgumentParser()
    parser.add_argument("target", type=str, help="監視対象ディレクトリ")
    args = parser.parse_args()

    target_dir = str(Path(args.target).resolve())
    if Path(target_dir).is_dir() is False:
        print(target_dir + " does not exist.")
        sys.exit(1)
    Main().run(target_dir)
